--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  menudem.pl
--
--**************************************************************************

module MenuDem is
--   illustrates how to create a nested menu using a Menu ADT
use Input             -- to get menu input
use WorkFile          -- to get active file's name
use WkSpace           -- to erase files from memory
use Cell              -- to update the cell A:A1
use Utility           -- for Beep
-- Prologue
integer MainChoice            -- used to record the main menu
                              -- item selected
integer FileChoice            -- used to record the file menu
                              -- item selected
string array [4] MainTitles   -- main menu titles
string array [4] MainDesc     -- main menu descriptions
string array [2] FileTitles   -- file menu titles
string array [2] FileDesc     -- file menu descriptions
string array [8] GenericHelp  -- help text for both menus
procedure InitMenus is
-- assigns values to menu arrays
   MainTitles [1]="File menu"
   MainTitles [2]="Stamp"
   MainTitles [3]="Ready"
   MainTitles [4]="Quit"
   MainDesc [1]="File Save, and Clear"
   MainDesc [2]="Record worksheet file name in cell A:A1"
   MainDesc [3]="Return to 1-2-3 READY mode"
   MainDesc [4]="Exit 1-2-3 even if worksheets not saved"
   FileTitles [1]="Save"
   FileTitles [2]="Clear"
   FileDesc [1]="Save a file to disk if it has changed"
   FileDesc [2]="Clear all worksheets and files from memory"
   -- sets up help menu
   GenericHelp [1]=""
   GenericHelp [2]="To select a command from the menu, " ||
                   "highlight the command and press"
   GenericHelp [3]="ENTER, or press the first character " ||
                   "of the command. When you"
   GenericHelp [4]="highlight a command, 1-2-3 displays an " ||
                   "explanation for that"
   GenericHelp [5]="command in the third line of the control panel."
   GenericHelp [6]=""
   GenericHelp [7]="Select READY from the main menu to quit " ||
                   "the menu. Press ESC"
   GenericHelp [8]="to exit the help screen."
   -- initialize the menu item to highlight
   MainChoice = 1
   FileChoice = 1
end procedure InitMenus
procedure StampOption is
-- saves the name of the current file in cell A:A1
   WorkFile [] CurrentFile
   Cell ["A:A1"] TopCell
   string Filename = CurrentFile.FullName
   if Filename = "" then
      TopCell.Value = "{NO NAME}"
   else
      TopCell.Value = Filename
   end if
end procedure StampOption
procedure SaveOption is
-- save the current worksheet file
   begin  -- signal block
       WorkFile [] FName
       if FName.Changed = Changed then
          Save(FName,BackUpFile)
       end if
   end  -- signal block
      except on others do
         -- beep to alert the user that the save didn't occur
         Beep(750,100)
      end except
end procedure SaveOption
procedure EraseOption is
-- erases all files in memory
   Workspace [] W
   Erase(W)
      except on others do
         -- do nothing
      end except
end procedure EraseOption
procedure Main is
-- displays menus and processes options, until user selects READY
   -- initialize the various menu string arrays
   InitMenus
   -- create corresponding Menu ADTs
   Menu MainMenu = Menu(MainTitles,MainDesc)
   Menu FileMenu = Menu(FileTitles,FileDesc)
   -- assign help arrays
   MainMenu.Help = GenericHelp
   FileMenu.Help = GenericHelp
   loop
      begin  -- signal block
         -- display the main menu
         MainChoice = Input (MainMenu)
         case MainChoice
            of 1: FileChoice = Input (FileMenu)
                  case FileChoice
                     of 1: SaveOption
                     of 2: EraseOption
                  end case
            of 2: StampOption
            -- end the menu
            of 3: exit
            -- invoke quit from menu
            of 4: MacroRun("/QYY")
         end case
      end  -- signal block
         except on Escape, Break do
            -- don't let user use ESC or CTRL-BREAK to quit menu
            continue
         end except
   end loop
end procedure Main
end module MenuDem
