/***************************************************************************\

   Copyright (C) 1989-1994   Lotus Development Corporation.
   All rights reserved.

   File Name:  newtrap.c

   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
       the source code version and to reproduce and distribute the object
       code version of the Example Programs provided in the Toolkit, provided
       that you:

       a) distribute the Example Programs only in conjunction with and as a
          part of your software application product which is designed as an
          add-in to Lotus 1-2-3, and

       b) do not represent your product to be a product of Lotus Development
          Corporation.

   2.  The Example Programs are provided as is, without warranty of any kind,
       either express or implied, including, without limitation, the implied
       warranties of merchantability of fitness for a particular purpose.

\***************************************************************************/

#define LPI_MAIN_MODULE

#include "newtrap.plh"
#include "lpixport.h"

#if defined(DEBUGON)
#include "debug.h"
#endif

LpiBindProto (newt_bind);
LpiUnbindProto (newt_unbind);
LpiProto (newton_raphson);
LpiProto (newton_raphson2);

void LpiIfInt(lfloat8, lptr(lfloat8));
void LpiIfAbs(lfloat8, lptr(lfloat8));

/****************************************************************************\
 *
 * Convert a double to an integer
 *
\****************************************************************************/
void LpiIfInt(mydouble, outnum)
   lfloat8        mydouble;
   lptr(lfloat8)  outnum;
{

   LpiPushLfloat8(&mydouble);
   LpiInt();
   LpiPopLfloat8(outnum);
}

/****************************************************************************\
 *
 * Convert a double to the absolute value of itself
 *
\****************************************************************************/
void LpiIfAbs(mydouble, outnum)
   lfloat8        mydouble;
   lptr(lfloat8)  outnum;
{

   LpiPushLfloat8(&mydouble);
   LpiAbs();
   LpiPopLfloat8(outnum);
}





/****************************************************************************\
 *
 * 8-byte infix implementation of Newton-Raphson method
 *
\****************************************************************************/
LpiDecl(newton_raphson)
{
   lfloat8  number;     /* number to find the root of */
   lfloat8  k;          /* holds floating-point value of index - 1 */
   lfloat8  guess[2];   /* guesses at root, should be almost equal when done */
   lfloat8  guep,       /* used for calculating guess[0] to index -1 */
            root;       /* the answer, when done */
   lslong   index,      /* integer root intended to find */
            j;          /* loop variable */
   lfloat8  tmp;

   IgnoreParameters;    /* avoid compiler error messages */

   index = LpiGetInInteger(1);      /* get input: root sought */
   number = LpiGetInDouble(0);      /* get input: number to find root of */

   guess[0] = 1.;       /* initialize guess array components */
   guess[1] = 1.;

   k = (lfloat8)index - 1.;   /* get lfloat8 value index - 1 */

   do {                       /* loop until guess[0] almost = guess[1] */
      /*substitute x for x sub zero */
      guess[0] = guess[1];

      guep = 1.;
      for (j=1; j < index; j++)/* calculate x sub zero to the index-1 power */
         guep *= guess[0];

      /* execute Newton-Raphson formula */
      guess[1] = (k * guess[0] + number/guep)/(lfloat8)index;

      LpiIfAbs(guess[1] - guess[0], &tmp);
   }
   while ( tmp >= .0000001 );

   /* Test for Epsilon (above).  A do-while loop is used to make the loop
    * execute once before evaluation of Epsilon.  Otherwise, the loop would
    * not execute at all, since both guess variables are initialized at
    * the same time.  Move one more than the desired number of decimal places
    * to the left of the decimal point.  Note multiplication by a float value.
    */

   /* Lop off any excess decimal places.  To make it easier,
    * multiply by one million (will divide by one million later).
    */
    root = guess[1] * 1000000.;

    /* Use integer division and multiplication to reduce units place to zero.
     * Round up, if appropriate.  The tens place in 'root' as it stands here
     * will become the last decimal place.
     */
    LpiIfInt(root/10, &tmp);
    if (root - tmp * 10 >= 5.)
         root = root + 10.;

   /* Since this is only accurate to 5 decimal places, eliminate decimal
    * places that are left after rounding.
    */
   LpiIfInt(root, &root);

   /* Return root to its correct value, but with last decimal place rounded.
    * Note division by a float value.
    */
   root = root / 1000000.;


   LpiSetOutDouble(0, root); /* return results to caller */

   LpiSignal( LPI_SUCCESS );  /* signal success and return */
}


/****************************************************************************\
 *
 * 10-byte stack-based implementation of Newton-Raphson method
 *
\****************************************************************************/
LpiDecl(newton_raphson2)
{
   lfloat10    number;  /* number to find the root of */
   lfloat10    k;       /* holds floating-point value of index - 1 */
   lfloat10    guess[2];/* guesses at root, should be almost equal when done */
   lfloat10    guep,    /* used for calculating guess[0] to index -1 */
               root;    /* the answer, when done */
   lslong      index,   /* integer root intended to find */
               j;       /* loop variable */
   lfloat10    testnum; /* number to test to see if root is valid */

   IgnoreParameters;    /* avoid compiler error messages */

   index = LpiGetInInteger(1);      /* get input: root sought */
   number = LpiGetInFloat(0);       /* get input: number to find root of */

   LpiPushOne();                 /* guess[0] = 1.; */
   LpiPopLfloat10(&guess[0]);
   LpiPushOne();                 /* guess[1] = 1.; */
   LpiPopLfloat10(&guess[1]);

   LpiPushLsLong(index);         /* k = index - 1.; */
   LpiPushOne();
   LpiSub();
   LpiPopLfloat10(&k);


   /* here is how we get a constant of .0000001 */
   LpiPushLsShort(10);
   LpiPushLsShort(-6);
   LpiPow();
   LpiPopLfloat10(&testnum);


   do {
                                       /*substitute x for x sub zero */
      LpiPushLfloat10(&guess[1]);      /* guess[0] = guess[1]; */
      LpiPopLfloat10(&guess[0]);

      /* calculate x sub zero to the index-1 power */
      LpiPushOne();            /*      guep = 1.; */
      for (j=1; j < index; j++) {
         LpiPushLfloat10(&guess[0]);   /* guep *= guess[0]; */
         LpiMul();
      }
      LpiPopLfloat10(&guep);


      /* execute Newton-Raphson formula */
      /* guess[1] = (k * guess[0] + number/guep)/(lfloat8)index; */
      LpiPushLfloat10(&k);          /* x = k * guess[0] */
      LpiPushLfloat10(&guess[0]);
      LpiMul();
      LpiPushLfloat10(&number);     /* y = number / guep */
      LpiPushLfloat10(&guep);
      LpiDiv();
      LpiAdd();                     /* z = x + y */
      LpiPushLsLong(index);
      LpiDiv();                     /* z / index */
      LpiPopLfloat10(&guess[1]);


      /* First part of test for Epsilon.  Set TOS to LTRUE if guess[1]
       * and guess[0] are within .0000001 of each other.
       */
      LpiPushLfloat10(&guess[1]);   /* guess[1] - guess[0] >= .0000001 */
      LpiPushLfloat10(&guess[0]);
      LpiSub();
      LpiAbs();                     /* take absolute value of difference */
      LpiPushLfloat10(&testnum);
      LpiGte();

   }
   while ( LpiPopLsShort() );    /* pops result of compare (1 if gte) */

   /* Test for Epsilon.  A do-while loop is used to make the loop execute
    * once before evaluation of Epsilon.  Otherwise, the loop would not
    * execute at all, since both guess variables are initialized at the
    * same time.  Move one more than the desired number of decimal places
    * to the left of the decimal point.  Note multiplication by a float value.
    */

   /* Lop off any excess decimal places.  To make it easier,
    * multiply by one million (will divide by one million later).
    */
    LpiPushLsLong( 1000000L );   /* root = guess[1] * 1000000.; */
    LpiPushLfloat10(&guess[1] );
    LpiMul();
    LpiPopLfloat10(&root);

    /* Use integer division and multiplication to reduce units place to zero.
     * Round up if appropriate.  The tens place in 'root' as it stands here
     * will become the last decimal place.
     */
    LpiPushLfloat10(&root);   /* for later subtraction (see next line) */
    LpiPushLfloat10(&root);   /* if (root - LpiIfInt(root/10) * 10 >= 5.) */
    LpiPushLsShort(10);
    LpiDiv();
    LpiInt();                 /* convert to integer */
    LpiPushLsShort(10);
    LpiMul();
    LpiSub();                 /* root - (....) */
    LpiPushLsShort(5);
    LpiGte();

    if (LpiPopLsShort()) {
       LpiPushLsShort(10);       /* root = root + 10.; */
       LpiPushLfloat10(&root);
       LpiAdd();
       LpiPopLfloat10(&root);
    }

   /* Since this is only accurate to 5 decimal places, eliminate decimal
    * places that are left after rounding.
    */
   LpiPushLfloat10(&root);
   LpiInt();
   /* Leave the result (root * 100,000) without decimal places on stack. */

   /* Return root to its correct value, but with last decimal place rounded.
    * Note division by a float value in 8 byte is accomplished with long here.
    */
   /* root = root / 1000000. */
   LpiPushLsLong(1000000L);
   LpiDiv();
   LpiPopLfloat10(&root);

   LpiSetOutFloat(0, root);      /* return results to caller */

   LpiSignal( LPI_SUCCESS );     /* return normally */
}



LpiBind(newt_bind)
{
   /* The following 4 lines set up a table of functions that can be called
    * from LPL.
    */
   LpiSetBegin
      LpiSet(newton_raphson)
      LpiSet(newton_raphson2)
   LpiSetEnd

   LpiSetCode(NEWTRAP_STAMP_0, NEWTRAP_STAMP_1, NEWTRAP_VERSION );


   LPI_SETUP_LOAD;   /* start loading pointers to used modules */

#if defined(DEBUGON)
   debug();
#endif

   LpiSignal(LPI_SUCCESS);    /* everything is ok */
}


/************************************************************************
 *
 * This is the unbind function.  It is called only when the user selects
 * ALT-F10 Remove or ALT-F10 Clear.  The code in this function works in
 * most circumstances.
 *
 ************************************************************************/

LpiUnbind(newt_unbind)
{
   LPI_UNLOAD;
   LpiSignal(LPI_SUCCESS);
}

