--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  prmptdem.pl
--
--**************************************************************************

module PrmptDem is
use Input
use Window      -- to display prompt window
use Utility     -- for procedure Beep
procedure BoxIt (Display Monitor, Prompt ThePrompt,
                String Msg, Ttle)
                returns (Window Box, Prompt ThePrompt)
                signals (Failed) is
   -- initializes the Window and Prompt objects that are
   -- passed as input.
   begin  -- signal block
      -- determine screen size
      integer DisplayCols, DisplayRows = Monitor.Size
      -- set input field width
      integer FieldWidth = 15
      -- check message length
      if Length(Msg) + FieldWidth < DisplayCols then
         string Label = Msg
      else
         -- abbreviate the string
         Label = Msg [1..DisplayCols - FieldWidth]
      end if
      -- Declare a Window ADT by explicitly calling a
      -- Window constructor
      -- procedure.  See Chapter 5 of the Language Reference for
      -- information on constructor procedures.
      Box = Window (DisplayCols - FieldWidth, 5, Single)
      -- set colors
      Box.Foreground (DefaultArea) = Normal
      Box.Background (DefaultArea) = Inverse
      -- set title and position
      Box.Title = Ttle
      Box.Position = FieldWidth div 2, (DisplayRows div 2) - 5
      -- set prompt position
      ThePrompt.Title = Label
      ThePrompt.Column = FieldWidth div 2 + 3
      ThePrompt.Row = DisplayRows div 2 - 3
      ThePrompt.EndColumn = DisplayCols - FieldWidth div 2 - 4
   end  -- signal block
      except on others do
         signal Failed
      end except
end procedure BoxIt
procedure BoxPrompt (string Msg, Ttle) returns (string Data)
                     signals (Failed) is
-- displays a prompt in a window in the center of the display
-- and returns the user input
   begin  -- signal block
      Display [] Monitor
      Window [] Box
      -- declare a Prompt ADT
      Prompt [] ThePrompt
      -- call BoxIt to initialize the Window and Prompt objects
      Box, ThePrompt = BoxIt (Monitor, ThePrompt, Msg, Ttle)
      -- display the window
      Box.Enabled = true
      -- get input
      Data = Input (ThePrompt)
   end  -- signal block
      except on others do
         signal Failed
      end except
end procedure BoxPrompt
procedure BoxPrompt (string Msg, Ttle) returns (float Data)
                    signals (Failed) is
-- displays a prompt in a window in the center of the display
-- and returns the user input
   begin  -- signal block
      Display [] Monitor
      Window [] Box
      -- declare a Prompt ADT
      Prompt [] ThePrompt
      -- call BoxIt to initialize the Window and Prompt objects
      Box, ThePrompt = BoxIt (Monitor, ThePrompt, Msg, Ttle)
      -- display the window
      Box.Enabled = true
      -- get input
      Data = Input (ThePrompt)
   end  -- signal block
      except on others do
         signal Failed
      end except
end procedure BoxPrompt
procedure BoxPrompt (string Msg, Ttle) returns (integer Data)
                    signals (Failed) is
-- displays a prompt in a window in the center of the display
-- and returns the user input
   begin  -- signal block
      Display [] Monitor
      Window [] Box
      -- declare a Prompt ADT
      Prompt [] ThePrompt
      -- call BoxIt to initialize the Window and Prompt objects
      Box, ThePrompt = BoxIt (Monitor, ThePrompt, Msg, Ttle)
      -- display the window
      Box.Enabled = true
      -- get input
      Data = Input (ThePrompt)
   end  -- signal block
      except on others do
         signal Failed
      end except
end procedure BoxPrompt
procedure GetInput (string Msg, Ttle, AnsCell) is
-- prompts a user for string input
   begin  -- signal block
      -- declare a Cell object to hold user input
      Cell [AnsCell] TheCell
      -- call BoxPrompt to get user input
      string Answer = BoxPrompt (Msg, Ttle)
      -- put user input in the answer cell
      TheCell.Contents = Answer
      -- refresh the window display that may have been overwritten
      -- by TheCell.Contents
      Refresh (Display, WorkSheet)
   end  -- signal block
      except on others do
         -- beep and end the procedure
         Beep (750, 100)
      end except
end procedure GetInput
procedure Execute is
-- prompts a user for an operating system command and
-- calls System to execute the command
   begin  -- signal block
       string Command =
            BoxPrompt ("Enter an operating system command ===> ",
            " SYSTEM ")
       if Command <> "" then
          System (Command)
       end if
       -- refresh the display
       Refresh (Display, WorkSheet)
   end  -- signal block
      except on others do
         -- ignore all signals
      end except
end procedure Execute
procedure GetInteger (string Msg, AnsCell, integer Min, Max) is
-- prompts a user for a number that falls in a specified range
   begin  -- signal block
      string Ttle = "Get Integer Value"
      Prompt [] ThePrompt
      -- set the minimum value
      ThePrompt.Min = Min
      -- set the maximum value
      ThePrompt.Max = Max
      -- create a help array
      string array [6] Help
      Help [1]=""
      Help [2]="Type a whole number in the range " ||
            string(Min) || " to " || string(Max) || ","
      Help [3]="and then press Enter. Alternatively, " ||
            "press the right and left"
      Help [4]="cursor keys, to rotate through the numbers."
      Help [5]=""
      Help [6]="Press ESC to continue."
      -- assign the help array
      ThePrompt.Help = Help
      Display [] Monitor
      Window [] Box
      -- call BoxIt to initialize the Window and Prompt objects
      Box, ThePrompt = BoxIt (Monitor, ThePrompt, Msg, Ttle)
      -- display the window
      Box.Enabled = true
      integer Result = Input (ThePrompt)
      Cell [AnsCell] TheCell
      -- save input value in cell
      TheCell.Value = Result
      Refresh (Display, WorkSheet)
   end  -- signal block
      except
         on others do
            -- beep and end the procedure
            Beep (750, 100)
      end except
end procedure GetInteger
procedure Main is
   begin  -- signal block
   -- prompts user for input and uses the LPL built-in procedure
   -- Lead to trim leading blanks from the response
      string Choice = Lead (BoxPrompt ("Do you want to Start [Y]? ",
            "Proceed Box"))
      -- keep prompting as long as the user types Y or y to continue
      while Choice like "Y" | Choice = "" loop
         Choice = Lead (BoxPrompt
               ("Do you want try a System Command [Y]? ",
               "System Command Example"))
         if Choice like "Y" | Choice = "" then
            Execute
         end if
         Choice = Lead (BoxPrompt
               ("Do you want enter a Label [Y]? ",
               "Label Input Example"))
         if Choice like "Y" | Choice = "" then
            string CellAddr = Lead (BoxPrompt
                  ("Enter location to store information: ",
                  "Label Input Example"))
            GetInput ("Enter Label Information: ",
                  "Label Input Example", CellAddr)
         end if
         Choice = Lead (BoxPrompt
               ("Do you want enter an Integer  [Y]? ",
               "Integer Input Example"))
         if Choice like "Y" | Choice = "" then
            CellAddr = Lead (BoxPrompt
                  ("Enter location to store information: ",
                  "Integer Input Example"))
            integer Min = BoxPrompt
                  ("Enter allowed Minimum Value: ",
                  "Integer Input Example")
            integer Max = BoxPrompt
                  ("Enter allowed Maximum Value: ",
                  "Integer Input Example")
            GetInteger ("Enter Integer in between " ||
                  string (Min) || ".." || string (Max) || ": ",
                  CellAddr, Min, Max)
         end if

         Choice = Lead (BoxPrompt ("Do you want to continue [Y]? ",
               "Proceed Box"))
      end loop
   end  -- signal block
      except
         on others do
            -- beep and end the procedure
            Beep (750, 100)
      end except
end procedure Main
end module PrmptDem
