--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  sheetdem.pl
--
--**************************************************************************

module SheetDem is
-- This add-in shows how to check a row or column for data before
-- deleting it, and how to get and set worksheet attributes.
use Sheet
use Cell    -- to check cell type
use Macro   -- to move cell pointer
use WkSpace -- to relocate cell pointer

procedure SafeColRowDel (integer Pos, DeleteType ID)
                        signals (DataFound, Failed) is
-- deletes the row or column if no data exists in the row or column
   begin -- signal block
      -- declare Workspace ADT
      Workspace [] Wspc
      -- declare Sheet ADT
      Sheet [] LiveSheet
      -- get worksheet protection
      boolean ProtectState = LiveSheet.Protection
      if ProtectState then
         -- turn off protection
         LiveSheet.Protection = false
      end if
      -- get cell pointer details
      string PtrName, integer PtrSht,PtrCol,PtrRow = Wspc.CellPointer
      -- if the DeleteType is Columns
      if ID = Columns then
         -- move to the top of the column
         Wspc.CellPointer = PtrName, PtrSht, Pos, 1
         Cell [] CellC
         -- check that the cell is not empty
         if CellC.CellType <> Blank then
            -- move the cell pointer back
            Wspc.CellPointer = PtrName, PtrSht, PtrCol, PtrRow
            signal DataFound
         end if
         -- move the cell pointer down to the next cell in the column
         -- that contains data
         MacroRun ("{End}{Down}")
         -- declare an ADT object for the current cell
         Cell [] TestRow
         -- if the cell pointer is not at the last row of the
         -- worksheet, then data was found
         if TestRow.Row <> 8192 then
            -- move the cell pointer back
            Wspc.CellPointer = PtrName, PtrSht, PtrCol, PtrRow
            signal DataFound
         end if
      else  -- (ID = Rows)
         -- move to the left of the row
         Wspc.CellPointer = PtrName, PtrSht, 1, Pos
         Cell [] CellR
         -- check that the cell is not empty
         if CellR.CellType <> Blank then
            -- move the cell pointer back
            Wspc.CellPointer = PtrName, PtrSht, PtrCol, PtrRow
            signal DataFound
         end if
         MacroRun ("{End}{Right}")
         -- declare an ADT object for the current cell
         Cell [] TestCol
         -- if the cell pointer is not at the column of the
         -- worksheet, then data was found
         if TestCol.Column <> 256 then
            -- move the cell pointer back
            Wspc.CellPointer = PtrName, PtrSht, PtrCol, PtrRow
            signal DataFound
         end if
      end if  -- (if ID = Columns)
      -- delete the row or column
      Delete (LiveSheet, ID, Pos, 1)
      -- move the cell pointer back
      Wspc.CellPointer = PtrName, PtrSht, PtrCol, PtrRow
      -- restore the protection status
      LiveSheet.Protection = ProtectState
   end   -- signal block
      except on others do
         -- fail procedure on any signal
         signal failed
      end except
end procedure SafeColRowDel
procedure AdjustSettings is
-- changes worksheet settings
   begin
      Sheet [] ActiveSheet
      -- get the column width
      integer CWidth = ActiveSheet.ColumnWidth
      -- if width is smaller or larger than allowed
      if CWidth < 5 or CWidth > 15 then
         -- adjust to 10
         ActiveSheet.ColumnWidth = 10
      end if
      -- get format
      FormatType Fmt, integer DecPlaces = ActiveSheet.Format
      -- check if the Format is Currency
      if Fmt <> Currency then
         -- if not, set it to Currency
         ActiveSheet.Format = Currency, 0
      end if
      -- center labels
      ActiveSheet.LabelPrefix = Center
      -- get the number of column titles
      integer FrozenCols = ActiveSheet.Titles(Vertical)
      -- get the number of row titles
      integer FrozenRows = ActiveSheet.Titles(Horizontal)
      -- if titles are not set
      if FrozenCols = 0 and FrozenRows = 0 then
         -- set them
         ActiveSheet.Titles (Both) = 1, 3
      end if
   end   -- signal block
   except on others do
   -- do nothing on exception
   end except
end procedure AdjustSettings

procedure Main is
-- Process the first 20 rows and the first 20 columns.
-- If the row/column is empty, delete it and check the newly
-- positioned row/column, otherwise move on to the adjacent
-- row/column.
   string Filename
   integer Lastsheet,Lastcol,Lastrow
   -- get the location of the {END}{HOME} cell for the active sheet
   Sheet [] ActiveSheet
   Filename,LastSheet,LastCol,LastRow = ActiveSheet.LastCell
   -- process the columns in the active part of the worksheet
   integer Col = 1
   while (Col <= LastCol) loop
      SafeColRowDel (Col, Columns)
         except
            on DataFound do
               -- if the column contains data, continue with next row
               Col = Col + 1
               continue
            on Failed do
               -- stop deleting columns and continue
               Exit
         end except
      -- decrement LastCol because a column was deleted
      LastCol = LastCol - 1
   end loop
   -- process the rows in the active part of the worksheet
   integer Row = 1
   while (Row <= LastRow) loop
      SafeColRowDel (Row, Rows)
         except
            on DataFound do
               -- the row has data, continue with the next row
               Row = Row + 1
               continue
            on Failed do
               -- stop deleting rows and continue
               Exit
         end except
      -- decrement LastRow because a row was deleted
      LastRow = LastRow - 1
   end loop
   -- call procedure AdjustSettings
   AdjustSettings
end procedure Main
end module SheetDem
